# pylint: disable=cyclic-import
# -*- coding: utf-8 -*-
#
#       ramstk.models.allocation.table.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Allocation Package Table Model."""

# Standard Library Imports
from typing import Any, Dict, Tuple, Type

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.analyses import allocation
from ramstk.models import RAMSTKAllocationRecord, RAMSTKBaseTable


class RAMSTKAllocationTable(RAMSTKBaseTable):
    """Contain the attributes and methods of the Allocation data manager."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _db_id_colname = "fld_hardware_id"
    _db_tablename = "ramstk_allocation"
    _select_msg = "selected_revision"
    _tag = "allocation"

    _system_hazard_rate: float = 0.0

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[str, Any]) -> None:
        """Initialize a Allocation data manager instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_id_columns = [
            "revision_id",
            "hardware_id",
        ]

        # Initialize private scalar attributes.
        self._node_hazard_rate: float = 0.0
        self._record: Type[RAMSTKAllocationRecord] = RAMSTKAllocationRecord

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.pkey = "hardware_id"

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self.do_calculate_agree_allocation, "request_calculate_agree_allocation"
        )
        pub.subscribe(
            self.do_calculate_arinc_allocation, "request_calculate_arinc_allocation"
        )
        pub.subscribe(
            self.do_calculate_equal_allocation, "request_calculate_equal_allocation"
        )
        pub.subscribe(
            self.do_calculate_foo_allocation, "request_calculate_foo_allocation"
        )
        pub.subscribe(
            self.do_calculate_allocation_goals, "request_calculate_allocation_goals"
        )

    def do_get_new_record(  # pylint: disable=method-hidden
        self, attributes: Dict[str, Any]
    ) -> object:
        """Gets a new record instance with attributes set.

        :param attributes: the dict of attribute values to assign to the new record.
        :return: None
        :rtype: None
        """
        # pylint: disable=attribute-defined-outside-init
        self._parent_id = attributes["parent_id"]

        _new_record = self._record()
        _new_record.revision_id = attributes["revision_id"]
        _new_record.parent_id = attributes["parent_id"]
        _new_record.hardware_id = attributes["hardware_id"]

        return _new_record

    def do_calculate_allocation_goals(self, node_id: int) -> None:
        """Calculate the allocation goals.

        :param node_id: the record ID whose goals are to be calculated.
        :return: None
        :rtype: None
        """
        _record = self.tree.get_node(node_id).data[self._tag]
        _attributes = _record.get_attributes()

        _attributes = allocation.do_calculate_goals(**_attributes)

        self.do_set_attributes(
            node_id=[node_id],
            package={"hazard_rate_goal": _attributes["hazard_rate_goal"]},
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"mtbf_goal": _attributes["mtbf_goal"]},
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"reliability_goal": _attributes["reliability_goal"]},
        )

    def do_calculate_agree_allocation(self, node_id: int, duty_cycle: float) -> None:
        """Allocate reliability using the AGREE method.

        :param node_id: the record ID whose allocation is to be calculated.
        :param duty_cycle: the duty cycle of the item to be calculated.
        :return: None
        :rtype: None
        """
        _record = self.tree.get_node(node_id).data[self._tag]
        _parent_goal = _record.reliability_goal
        _method_id = _record.allocation_method_id

        for _node in self.tree.children(node_id):
            _attributes: Dict[str, Any] = _node.data["allocation"].get_attributes()
            _attributes["allocation_method_id"] = _method_id
            _attributes["duty_cycle"] = duty_cycle

            (
                _attributes["n_sub_elements"],
                _attributes["n_sub_systems"],
            ) = self._do_calculate_agree_total_elements(node_id)

            _attributes = allocation.do_allocate_reliability(
                _parent_goal, 0, **_attributes
            )

            _node.data["allocation"].hazard_rate_alloc = _attributes[
                "hazard_rate_alloc"
            ]
            _node.data["allocation"].mtbf_alloc = _attributes["mtbf_alloc"]
            _node.data["allocation"].reliability_alloc = _attributes[
                "reliability_alloc"
            ]

            _parent_goal = _node.data["allocation"].reliability_goal

        pub.sendMessage(
            "succeed_calculate_allocation",
            tree=self.tree,
        )

    def do_calculate_arinc_allocation(self, node_id: int) -> None:
        """Allocate reliability using the ARINC method.

        :param node_id: the record ID whose allocation is to be calculated.
        :return: None
        :rtype: None
        :raises: ZeroDivisionError if the system hazard rate is zero.
        """
        _record = self.tree.get_node(node_id).data[self._tag]

        _parent_goal = _record.hazard_rate_goal
        _method_id = _record.allocation_method_id

        for _node in self.tree.children(node_id):
            _attributes = _node.data["allocation"].get_attributes()
            _attributes["allocation_method_id"] = _method_id

            _attributes["weight_factor"] = (
                self._node_hazard_rate / self._system_hazard_rate
            )
            _attributes = allocation.do_allocate_reliability(
                _parent_goal, 0, **_attributes
            )

            _node.data["allocation"].mtbf_alloc = _attributes["mtbf_alloc"]
            _node.data["allocation"].hazard_rate_alloc = _attributes[
                "hazard_rate_alloc"
            ]
            _node.data["allocation"].reliability_alloc = _attributes[
                "reliability_alloc"
            ]

            _parent_goal = _node.data["allocation"].hazard_rate_goal

        pub.sendMessage(
            "succeed_calculate_allocation",
            tree=self.tree,
        )

    def do_calculate_equal_allocation(self, node_id: int) -> None:
        """Allocate reliability using equal apportionment.

        :param node_id: the record ID whose allocation is to be calculated.
        :return: None
        :rtype: None
        """
        _record = self.tree.get_node(node_id).data[self._tag]
        _attributes = _record.get_attributes()

        _parent_goal = _attributes["reliability_goal"]
        _method_id = _attributes["allocation_method_id"]

        for _node in self.tree.children(node_id):
            _attributes = _node.data["allocation"].get_attributes()
            _attributes["allocation_method_id"] = _method_id

            _attributes["weight_factor"] = 1.0 / _node.data["allocation"].n_sub_systems
            _attributes = allocation.do_allocate_reliability(
                _parent_goal, 0, **_attributes
            )

            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"allocation_method_id": _attributes["allocation_method_id"]},
            )
            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"weight_factor": _attributes["weight_factor"]},
            )
            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"mtbf_alloc": _attributes["mtbf_alloc"]},
            )
            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"hazard_rate_alloc": _attributes["hazard_rate_alloc"]},
            )
            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"reliability_alloc": _attributes["reliability_alloc"]},
            )

            _parent_goal = _node.data["allocation"].reliability_goal

        pub.sendMessage(
            "succeed_calculate_allocation",
            tree=self.tree,
        )

    def do_calculate_foo_allocation(self, node_id: int) -> None:
        """Allocate reliability using the FOO method.

        :param node_id: the record ID whose allocation is to be calculated.
        :return: None
        :rtype: None
        """
        _record = self.tree.get_node(node_id).data[self._tag]

        _parent_goal = _record.hazard_rate_goal
        _method_id = _record.allocation_method_id
        _cum_weight = self._do_calculate_foo_cumulative_weight(node_id)

        for _node in self.tree.children(node_id):
            _attributes = _node.data["allocation"].get_attributes()
            _attributes["allocation_method_id"] = _method_id

            _attributes = allocation.do_allocate_reliability(
                _parent_goal, _cum_weight, **_attributes
            )

            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"weight_factor": _attributes["weight_factor"]},
            )
            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"percent_weight_factor": _attributes["percent_weight_factor"]},
            )
            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"mtbf_alloc": _attributes["mtbf_alloc"]},
            )
            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"hazard_rate_alloc": _attributes["hazard_rate_alloc"]},
            )
            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"reliability_alloc": _attributes["reliability_alloc"]},
            )

        pub.sendMessage(
            "succeed_calculate_allocation",
            tree=self.tree,
        )

    def _do_calculate_agree_total_elements(self, node_id: int) -> Tuple[int, int]:
        """Calculate the total number of elements for the AGREE method.

        :param node_id: the record ID whose allocation is to be calculated.
        :return: _n_sub_elements, _n_sub_systems; the number of
            sub-elements and subsystems comprising the selected node.
        :rtype: tuple
        """
        _n_sub_elements = 0
        _n_sub_systems = 0

        for _node in self.tree.children(node_id):
            _n_sub_elements += _node.data["allocation"].n_sub_elements
            _n_sub_systems += _node.data["allocation"].n_sub_systems

        return _n_sub_elements, _n_sub_systems

    def _do_calculate_foo_cumulative_weight(self, node_id: int) -> int:
        """Calculate the cumulative weight for the FOO method.

        :param node_id: the node (allocation) ID of the allocation item whose
            goal is to be allocated.
        :return: _cum_weight; the cumulative weighting factor for the
            allocation item to be allocated.
        """
        _cum_weight = 0

        for _node in self.tree.children(node_id):
            _attributes = _node.data["allocation"].get_attributes()
            _attributes["weight_factor"] = (
                _attributes["int_factor"]
                * _attributes["soa_factor"]
                * _attributes["op_time_factor"]
                * _attributes["env_factor"]
            )
            _cum_weight += _attributes["weight_factor"]

            self.do_set_attributes(
                node_id=[_node.identifier],
                package={"weight_factor": _attributes["weight_factor"]},
            )

        return _cum_weight



# -*- coding: utf-8 -*-
#
#       ramstk.models.program_status.table.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Program Status Table Model."""

# Standard Library Imports
from datetime import date
from typing import Any, Dict, List, Type

# Third Party Imports
import pandas as pd
from pubsub import pub

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseTable, RAMSTKProgramStatusRecord


class RAMSTKProgramStatusTable(RAMSTKBaseTable):
    """Contain the attributes and methods of the Program Status data manager."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _db_id_colname = "fld_status_id"
    _db_tablename = "ramstk_program_status"
    _select_msg = "selected_revision"
    _tag = "program_status"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[Any, Any]) -> None:
        """Initialize a Program Status data manager instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.
        self._dic_status: Dict[Any, List[float]] = {}

        # Initialize private list attributes.
        self._lst_id_columns = [
            "revision_id",
            "status_id",
        ]

        # Initialize private scalar attributes.
        self._record: Type[RAMSTKProgramStatusRecord] = RAMSTKProgramStatusRecord

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.pkey = "status_id"

        # Subscribe to PyPubSub messages.
        pub.subscribe(self.do_get_actual_status, "request_get_actual_status")
        pub.subscribe(self._do_set_attributes, "succeed_calculate_all_validation_tasks")

    def do_get_new_record(  # pylint: disable=method-hidden
        self, attributes: Dict[str, Any]
    ) -> object:
        """Gets a new record instance with attributes set.

        :param attributes: the dict of attribute values to assign to the new record.
        :return: None
        :rtype: None
        """
        _new_record = self._record()
        _new_record.revision_id = attributes["revision_id"]
        _new_record.status_id = self.last_id + 1
        _new_record.date_status = date.today()

        self._dic_status[_new_record.date_status] = _new_record.status_id

        return _new_record

    def do_get_actual_status(self) -> None:
        """Select the actual program status remaining time and cost.

        :return: a pandas DataFrame() containing the actual status update
            dates and the remaining time/cost.
        :rtype: :class:`pandas.DataFrame`
        """
        _dic_actual = {}

        for _node in self.tree.all_nodes()[1:]:
            _dic_actual[pd.to_datetime(_node.data["program_status"].date_status)] = [
                _node.data["program_status"].cost_remaining,
                _node.data["program_status"].time_remaining,
            ]

        _status = pd.DataFrame(
            _dic_actual.values(), index=_dic_actual.keys(), columns=["cost", "time"]
        ).sort_index()

        pub.sendMessage("succeed_get_actual_status", status=_status)

    def _do_set_attributes(self, cost_remaining, time_remaining) -> None:
        """Set the program remaining cost and time.

        :param cost_remaining: total remaining cost of verification program.
        :param time_remaining: total remaining time of verification program.
        :return: None
        :rtype: None
        """
        try:
            _node_id = self._dic_status[date.today()]
        except KeyError:
            self.do_insert(
                attributes={
                    "revision_id": self._revision_id,
                    "status_id": -1,
                    "date_status": date.today(),
                    "cost_remaining": cost_remaining,
                    "time_remaining": time_remaining,
                }
            )
            _node_id = self.last_id

        self.do_set_attributes(
            node_id=[_node_id], package={"cost_remaining": cost_remaining}
        )
        self.do_set_attributes(
            node_id=[_node_id], package={"time_remaining": time_remaining}
        )

        self.do_update(_node_id)



# pylint: disable=unused-import
"""The RAMSTK Analyses Package."""

# RAMSTK Local Imports
from . import (
    allocation,
    criticality,
    derating,
    dormancy,
    fha,
    improvementfactor,
    similaritem,
    stress,
)



# Standard Library Imports
from typing import Any, Callable, Dict, List, Tuple

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel

class HazardsTreePanel(RAMSTKTreePanel):
    _select_msg: str
    _tag: str
    _title: str
    _dic_row_loader: Dict[str, Callable]
    _filtered_tree: bool
    _on_edit_message: str
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def filter_tree(
        self, model: Gtk.TreeModel, row: Gtk.TreeIter, data: Any
    ) -> bool: ...
    def do_load_severity(
        self, criticalities: Dict[int, Tuple[str, str, int]]
    ) -> None: ...
    def do_load_hazards(self, hazards: Dict[Any, Any]) -> None: ...
    def do_load_probability(self, probabilities: List[str]) -> None: ...
    _parent_id: int
    _record_id: int
    def _on_row_change(self, selection: Gtk.TreeSelection) -> None: ...
    def _on_select_function(self, attributes: Dict[str, Any]) -> None: ...



# Standard Library Imports
from typing import Any, Callable, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKCheckButton as RAMSTKCheckButton
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel
from ramstk.views.gtk3.widgets import RAMSTKTextView as RAMSTKTextView
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel

class FunctionTreePanel(RAMSTKTreePanel):
    _select_msg: str
    _tag: str
    _title: str
    _type: str
    _dic_row_loader: Dict[str, Callable]
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _on_module_switch(self, module: str = ...) -> None: ...
    _record_id: int
    _parent_id: int
    def _on_row_change(self, selection: Gtk.TreeSelection) -> None: ...

class FunctionGeneralDataPanel(RAMSTKFixedPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    _type: str
    chkSafetyCritical: RAMSTKCheckButton
    txtCode: RAMSTKEntry
    txtName: RAMSTKEntry
    txtRemarks: RAMSTKEntry
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.models.function.table.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Function Table Model."""

# Standard Library Imports
from typing import Any, Dict, Type

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseTable, RAMSTKFunctionRecord


class RAMSTKFunctionTable(RAMSTKBaseTable):
    """Contain the attributes and methods of the Function table model."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _db_id_colname = "fld_function_id"
    _db_tablename = "ramstk_function"
    _select_msg = "selected_revision"
    _tag = "function"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[Any, Any]) -> None:
        """Initialize a Function data manager instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_id_columns = [
            "revision_id",
            "function_id",
            "parent_id",
            "record_id",
        ]

        # Initialize private scalar attributes.
        self._record: Type[RAMSTKFunctionRecord] = RAMSTKFunctionRecord

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.pkey = "function_id"

        # Subscribe to PyPubSub messages.

    def do_get_new_record(  # pylint: disable=method-hidden
        self, attributes: Dict[str, Any]
    ) -> object:
        """Gets a new record instance with attributes set.

        :param attributes: the dict of attribute values to assign to the new record.
        :return: None
        :rtype: None
        """
        self._parent_id = attributes[  # pylint: disable=attribute-defined-outside-init
            "parent_id"
        ]

        _new_record = self._record()
        _new_record.revision_id = attributes["revision_id"]
        _new_record.function_id = self.last_id + 1
        _new_record.parent_id = attributes["parent_id"]

        return _new_record



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel

class ConnectionDesignElectricInputPanel(RAMSTKFixedPanel):
    _dic_quality: Dict[int, List[List[str]]]
    _dic_type: Dict[int, List[List[str]]]
    _dic_specification: Dict[int, List[List[str]]]
    _lst_insert_A: List[List[str]]
    _lst_insert_B: List[List[str]]
    _lst_insert_C: List[List[str]]
    _lst_insert_D: List[List[str]]
    _dic_insert: Dict[int, Dict[int, List[List[str]]]]
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    cmbInsert: RAMSTKComboBox
    cmbQuality: RAMSTKComboBox
    cmbSpecification: RAMSTKComboBox
    cmbType: RAMSTKComboBox
    txtContactGauge: RAMSTKEntry
    txtActivePins: RAMSTKEntry
    txtAmpsContact: RAMSTKEntry
    txtMating: RAMSTKEntry
    txtNWave: RAMSTKEntry
    txtNHand: RAMSTKEntry
    txtNPlanes: RAMSTKEntry
    _hazard_rate_method_id: int
    _quality_id: int
    dic_attribute_widget_map: Dict[str, List[Any]]
    category_id: int
    subcategory_id: int
    def __init__(self) -> None: ...
    def do_load_comboboxes(self, subcategory_id: int) -> None: ...
    def _do_load_insert(self, combo: RAMSTKComboBox) -> None: ...
    def _do_load_specification(self, combo: RAMSTKComboBox) -> None: ...
    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None: ...
    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def __do_set_circular_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def __do_set_ic_socket_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def __do_set_pwa_edge_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def __do_set_pth_sensitive(self, attributes: Dict[str, Any]) -> None: ...



# Standard Library Imports
from typing import Any, Callable, Dict, List

# Third Party Imports
import treelib

# RAMSTK Package Imports
from ramstk.views.gtk3 import GdkPixbuf as GdkPixbuf
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel

class PoFTreePanel(RAMSTKTreePanel):
    _dic_column_masks: Dict[str, List[bool]]
    _dic_headings: Dict[str, List[str]]
    _select_msg: str
    _tag: str
    _title: str
    _dic_row_loader: Dict[str, Callable]
    _dic_visible_mask: Dict[str, List[str]]
    _on_edit_message: str
    dic_attribute_widget_map: Dict[str, List[Any]]
    dic_icons: Dict[str, str]
    lst_damage_models: List[str]
    lst_load_history: List[str]
    lst_measurable_parameters: List[str]
    def __init__(self) -> None: ...
    def do_load_comboboxes(self) -> None: ...
    def _on_row_change(self, selection: Gtk.TreeSelection) -> None: ...
    def __do_load_damage_models(self) -> None: ...
    def __do_load_load_history(self) -> None: ...
    def __do_load_measureable_parameters(self) -> None: ...
    def __do_load_mechanism(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...
    def __do_load_mode(self, node: treelib.Node, row: Gtk.TreeIter) -> Gtk.TreeIter: ...
    def __do_load_opload(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...
    def __do_load_opstress(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...
    def __do_load_test_method(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...



# Standard Library Imports
from typing import Any, Dict

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseTable as RAMSTKBaseTable
from ramstk.models import RAMSTKNSWCRecord as RAMSTKNSWCRecord

class RAMSTKNSWCTable(RAMSTKBaseTable):
    _db_id_colname: str
    _db_tablename: str
    _select_msg: str
    _tag: str
    _lst_id_columns: Any
    _record: Any
    pkey: str
    def __init__(self, **kwargs: Dict[Any, Any]) -> None: ...
    def do_get_new_record(self, attributes: Dict[str, Any]) -> object: ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hazard_analysis.view.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Hazard Analysis Views."""

# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import RAMSTKPanel, RAMSTKWorkView

# RAMSTK Local Imports
from . import HazardsTreePanel


class HazardsWorkView(RAMSTKWorkView):
    """Display HazOps attribute data in the Work Book.

    The WorkView displays all the attributes for the Hazards Analysis (HazOps).
    The attributes of a HazOps Work View are:

    :cvar str _tag: the name of the module.

    :ivar list _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar list _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar list _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar list _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    :ivar int _hazard_id: the ID of the currently selected hazard.
    """

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag = "hazard"
    _tablabel = _("HazOps")
    _tabtooltip = _("Displays the HazOps analysis for the selected Function.")

    # Define public dict class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Work View for the HazOps.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_callbacks.insert(0, super().do_request_insert_sibling)
        self._lst_callbacks.insert(1, self.do_request_delete)
        self._lst_callbacks.insert(2, self._do_request_calculate)
        self._lst_icons.insert(0, "add")
        self._lst_icons.insert(1, "remove")
        self._lst_icons.insert(2, "calculate")
        self._lst_mnu_labels = [
            _("Add Hazard"),
            _("Delete Selected Hazard"),
            _("Calculate HazOp"),
            _("Save Selected Hazard"),
            _("Save All Hazards"),
        ]
        self._lst_tooltips = [
            _("Add a new hazard to the HazOps analysis."),
            _("Delete the selected hazard from the selected function."),
            _("Calculate the HazOps analysis."),
            _("Save changes to the selected hazard."),
            _("Save changes to all hazards."),
        ]

        # Initialize private scalar attributes.
        self._pnlPanel: RAMSTKPanel = HazardsTreePanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, "selected_hazard")
        pub.subscribe(self._on_select_function, "selected_function")

    def _do_request_calculate(self, __button: Gtk.ToolButton) -> None:
        """Request to calculate the HazOps HRI.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        :rtype: None
        """
        super().do_set_cursor_busy()
        pub.sendMessage("request_calculate_fha", node_id=self.dic_pkeys["record_id"])

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the record ID when a hazard is selected.

        :param attributes: the hazard dict for the selected hazard ID.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["revision_id"] = attributes["revision_id"]
        self.dic_pkeys["hazard_id"] = attributes["hazard_id"]
        self.dic_pkeys["record_id"] = attributes["hazard_id"]

    def _on_select_function(self, attributes: Dict[str, Any]) -> None:
        """Set the parent ID when a function is selected.

        :param attributes: the function dict for the selected function ID.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["parent_id"] = attributes["function_id"]
        self.dic_pkeys["revision_id"] = attributes["revision_id"]
        self.dic_pkeys["function_id"] = attributes["function_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the Function Hazard Analysis tab.

        :return: None
        :rtype: None
        """
        super().do_make_layout()
        super().do_embed_treeview_panel()

        self._pnlPanel.do_load_severity(self.RAMSTK_USER_CONFIGURATION.RAMSTK_SEVERITY)
        self._pnlPanel.do_load_hazards(self.RAMSTK_USER_CONFIGURATION.RAMSTK_HAZARDS)
        self._pnlPanel.do_load_probability(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_FAILURE_PROBABILITY
        )

        self.show_all()



# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
import pandas as pd

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseTable as RAMSTKBaseTable
from ramstk.models import RAMSTKValidationRecord as RAMSTKValidationRecord

class RAMSTKValidationTable(RAMSTKBaseTable):
    _db_id_colname: str
    _db_tablename: str
    _select_msg: str
    _tag: str
    _dic_status: Any
    _lst_id_columns: Any
    _record: Any
    pkey: str
    def __init__(self, **kwargs: Dict[Any, Any]) -> None: ...
    def do_get_new_record(self, attributes: Dict[str, Any]) -> object: ...
    def do_calculate_plan(self) -> None: ...
    def _do_calculate_all_tasks(self) -> None: ...
    def _do_calculate_task(self, node_id: int) -> None: ...
    def _do_select_assessment_targets(self) -> pd.DataFrame: ...



# -*- coding: utf-8 -*-
#
#       ramstk.models.programdb.hazard.table.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Hazard Package Table Model."""

# Standard Library Imports
from collections import OrderedDict
from typing import Any, Dict, Type

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.analyses import fha
from ramstk.models import RAMSTKBaseTable, RAMSTKHazardRecord


class RAMSTKHazardTable(RAMSTKBaseTable):
    """Contain the attributes and methods of the Hazard data manager."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _db_id_colname = "fld_hazard_id"
    _db_tablename = "ramstk_hazard_analysis"
    _select_msg = "selected_revision"
    _tag = "hazard"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[Any, Any]) -> None:
        """Initialize a Hazard data manager instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_id_columns = [
            "revision_id",
            "function_id",
            "hazard_id",
            "parent_id",
            "record_id",
        ]

        # Initialize private scalar attributes.
        self._record: Type[RAMSTKHazardRecord] = RAMSTKHazardRecord

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.pkey = "hazard_id"

        # Subscribe to PyPubSub messages.
        pub.subscribe(self.do_calculate_fha, "request_calculate_fha")

    def do_get_new_record(  # pylint: disable=method-hidden
        self, attributes: Dict[str, Any]
    ) -> RAMSTKHazardRecord:
        """Gets a new record instance with attributes set.

        :param attributes: the dict of attribute values to assign to the new record.
        :return: None
        :rtype: None
        """
        attributes["hazard_id"] = self.last_id + 1
        attributes["parent_id"] = attributes["function_id"]
        attributes["record_id"] = attributes["hazard_id"]

        _new_record = self._record()
        _new_record.revision_id = attributes["revision_id"]
        _new_record.function_id = attributes["function_id"]
        _new_record.hazard_id = attributes["hazard_id"]

        return _new_record

    def do_calculate_fha(self, node_id: int) -> None:
        """Perform a hazards analysis calculation for currently selected item.

        :param node_id: the ID of the record to calculate.
        :return: None
        :rtype: None
        """
        self._do_calculate_hri(node_id)
        self._do_calculate_user_defined(node_id)

        pub.sendMessage(
            f"succeed_calculate_{self._tag}",
            tree=self.tree,
        )

    def _do_calculate_hri(self, node_id: int) -> None:
        """Calculate the hazard risk index (HRI).

        This method calculates the assembly and system level HRI for both
        before and after mitigation actions.

        :param node_id: the ID of the record to calculate.
        :return: None
        :rtype: None
        :raises: KeyError if one or more attribute keys is missing.
        """
        _record = self.tree.get_node(node_id).data[self._tag]
        _attributes = _record.get_attributes()

        _result = fha.calculate_hri(
            _attributes["assembly_probability"],
            _attributes["assembly_severity"],
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"assembly_hri": _result},
        )

        _result = fha.calculate_hri(
            _attributes["system_probability"],
            _attributes["system_severity"],
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"system_hri": _result},
        )

        _result = fha.calculate_hri(
            _attributes["assembly_probability_f"],
            _attributes["assembly_severity_f"],
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"assembly_hri_f": _result},
        )

        _result = fha.calculate_hri(
            _attributes["system_probability_f"],
            _attributes["system_severity_f"],
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"system_hri_f": _result},
        )

    def _do_calculate_user_defined(self, node_id: int) -> None:
        """Calculate the user-defined hazard analysis.

        :param node_id: the ID of the record to calculate.
        :return: None
        :rtype: None
        """
        _record = self.tree.get_node(node_id).data[self._tag]
        _attributes = _record.get_attributes()

        _fha = OrderedDict(
            {
                _key: ""
                for _key in [
                    "uf1",
                    "uf2",
                    "uf3",
                    "ui1",
                    "ui2",
                    "ui3",
                    "equation1",
                    "equation2",
                    "equation3",
                    "equation4",
                    "equation5",
                    "res1",
                    "res2",
                    "res3",
                    "res4",
                    "res5",
                ]
            }
        )

        _fha = fha.set_user_defined_floats(
            _fha,
            [
                _attributes["user_float_1"],
                _attributes["user_float_2"],
                _attributes["user_float_3"],
            ],
        )

        _fha = fha.set_user_defined_ints(
            _fha,
            [
                _attributes["user_int_1"],
                _attributes["user_int_2"],
                _attributes["user_int_3"],
            ],
        )

        _fha = fha.set_user_defined_functions(
            _fha,
            [
                _attributes["function_1"],
                _attributes["function_2"],
                _attributes["function_3"],
                _attributes["function_4"],
                _attributes["function_5"],
            ],
        )

        _fha = fha.set_user_defined_results(
            _fha,
            [
                _attributes["result_1"],
                _attributes["result_2"],
                _attributes["result_3"],
                _attributes["result_4"],
                _attributes["result_5"],
            ],
        )

        _fha = fha.calculate_user_defined(_fha)

        self.do_set_attributes(
            node_id=[node_id],
            package={"result_1": float(_fha["res1"])},
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"result_2": float(_fha["res2"])},
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"result_3": float(_fha["res3"])},
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"result_4": float(_fha["res4"])},
        )
        self.do_set_attributes(
            node_id=[node_id],
            package={"result_5": float(_fha["res5"])},
        )



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel

class ReliabilityInputPanel(RAMSTKFixedPanel):
    _select_msg: str
    _tag: str
    _title: str
    cmbFailureDist: RAMSTKComboBox
    cmbHRMethod: RAMSTKComboBox
    cmbHRType: RAMSTKComboBox
    txtAddAdjFactor: RAMSTKEntry
    txtFailLocation: RAMSTKEntry
    txtFailScale: RAMSTKEntry
    txtFailShape: RAMSTKEntry
    txtMultAdjFactor: RAMSTKEntry
    txtSpecifiedHt: RAMSTKEntry
    txtSpecifiedHtVar: RAMSTKEntry
    txtSpecifiedMTBF: RAMSTKEntry
    txtSpecifiedMTBFVar: RAMSTKEntry
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def do_load_hr_distributions(self, distributions: List[str]) -> None: ...
    def do_load_hr_methods(self, methods: List[str]) -> None: ...
    def do_load_hr_types(self, hr_types: List[str]) -> None: ...
    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def _do_set_sensitive_assessed(self, type_id: int) -> None: ...
    def _do_set_sensitive_specified_ht(self, type_id: int) -> None: ...
    def _do_set_sensitive_specified_mtbf(self, type_id: int) -> None: ...
    def _do_set_sensitive_specified_distribution(self, type_id: int) -> None: ...

class ReliabilityResultsPanel(RAMSTKFixedPanel):
    _select_msg: str
    _tag: str
    _title: str
    txtActiveHt: RAMSTKEntry
    txtActiveHtVar: RAMSTKEntry
    txtDormantHt: RAMSTKEntry
    txtDormantHtVar: RAMSTKEntry
    txtLogisticsHt: RAMSTKEntry
    txtLogisticsHtVar: RAMSTKEntry
    txtLogisticsMTBF: RAMSTKEntry
    txtLogisticsMTBFVar: RAMSTKEntry
    txtLogisticsRt: RAMSTKEntry
    txtLogisticsRtVar: RAMSTKEntry
    txtMissionHt: RAMSTKEntry
    txtMissionHtVar: RAMSTKEntry
    txtMissionMTBF: RAMSTKEntry
    txtMissionMTBFVar: RAMSTKEntry
    txtMissionRt: RAMSTKEntry
    txtMissionRtVar: RAMSTKEntry
    txtPercentHt: RAMSTKEntry
    txtSoftwareHt: RAMSTKEntry
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _do_load_entries(self, attributes: Dict[str, Any]) -> None: ...
    def _do_load_entries_hazard_rate(self, attributes: Dict[str, Any]) -> None: ...
    def _do_load_entries_mtbf(self, attributes: Dict[str, Any]) -> None: ...
    def _do_load_entries_reliability(self, attributes: Dict[str, Any]) -> None: ...
    def __do_nudge_widgets(self) -> None: ...

class AvailabilityResultsPanel(RAMSTKFixedPanel):
    _select_msg: str
    _tag: str
    _title: str
    txtCostFailure: RAMSTKEntry
    txtCostHour: RAMSTKEntry
    txtLogisticsAt: RAMSTKEntry
    txtLogisticsAtVar: RAMSTKEntry
    txtMissionAt: RAMSTKEntry
    txtMissionAtVar: RAMSTKEntry
    txtPartCount: RAMSTKEntry
    txtTotalCost: RAMSTKEntry
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _do_load_entries(self, attributes: Dict[str, Any]) -> None: ...
    def _do_load_entries_hardware(self, attributes: Dict[str, Any]) -> None: ...
    def __do_nudge_widgets(self) -> None: ...



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.opload.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Operating Load Record Model."""

# Third Party Imports
from sqlalchemy import Column, ForeignKeyConstraint, Integer, String
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKOpLoadRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent table ramstk_op_load in the RAMSTK Program database.

    This table shares a Many-to-One relationship with ramstk_mechanism. This
    table shares a One-to-Many relationship with ramstk_op_stress. This table
    shares a One-to-Many relationship with ramstk_test_method.
    """

    __defaults__ = {"description": "", "damage_model": "", "priority_id": 0}
    __tablename__ = "ramstk_op_load"
    __table_args__ = (
        ForeignKeyConstraint(
            ["fld_revision_id", "fld_hardware_id", "fld_mode_id", "fld_mechanism_id"],
            [
                "ramstk_mechanism.fld_revision_id",
                "ramstk_mechanism.fld_hardware_id",
                "ramstk_mechanism.fld_mode_id",
                "ramstk_mechanism.fld_mechanism_id",
            ],
        ),
        {"extend_existing": True},
    )

    revision_id = Column("fld_revision_id", Integer, primary_key=True, nullable=False)
    hardware_id = Column(
        "fld_hardware_id", Integer, primary_key=True, default=-1, nullable=False
    )
    mode_id = Column("fld_mode_id", Integer, primary_key=True, nullable=False)
    mechanism_id = Column("fld_mechanism_id", Integer, primary_key=True, nullable=False)
    load_id = Column(
        "fld_load_id", Integer, primary_key=True, autoincrement=True, nullable=False
    )

    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    damage_model = Column(
        "fld_damage_model", String(512), default=__defaults__["damage_model"]
    )
    priority_id = Column(
        "fld_priority_id", Integer, default=__defaults__["priority_id"]
    )

    # Define the relationships to other tables in the RAMSTK Program database.
    mechanism: relationship = relationship(
        "RAMSTKMechanismRecord",
        back_populates="op_load",
    )
    op_stress: relationship = relationship(
        "RAMSTKOpStressRecord",
        back_populates="op_load",
        cascade="all,delete",
    )
    test_method: relationship = relationship(
        "RAMSTKTestMethodRecord",
        back_populates="op_load",
        cascade="all,delete",
    )

    is_mode = False
    is_mechanism = False
    is_opload = True
    is_opstress = False
    is_testmethod = False

    def get_attributes(self):
        """Retrieve current values of the RAMSTKOpLoad data model attributes.

        :return: {mechanism_id, load_id, description, damage_model,
                  priority_id} pairs
        :rtype: dict
        """
        _attributes = {
            "mechanism_id": self.mechanism_id,
            "load_id": self.load_id,
            "description": self.description,
            "damage_model": self.damage_model,
            "priority_id": self.priority_id,
        }

        return _attributes



# pylint: disable=non-parent-init-called
# -*- coding: utf-8 -*-
#
#       ramstk.gui.gtk.ramstk.Label.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTK Label Module."""

# Standard Library Imports
from typing import Any, Dict, List, Tuple

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk

# RAMSTK Local Imports
from .widget import RAMSTKWidget


class RAMSTKLabel(Gtk.Label, RAMSTKWidget):
    """The RAMSTK Label class."""

    # Define private class scalar attributes.
    _default_height = 25
    _default_width = 190

    def __init__(self, text: str) -> None:
        """Create RAMSTKLabel widget.

        :param text: the text to display in the label.
        """
        RAMSTKWidget.__init__(self)

        self.set_markup("<span>" + text + "</span>")
        self.show_all()

    def get_attribute(self, attribute: str) -> Any:
        """Get the value of the requested attribute.

        :param attribute: the name of the attribute to retrieve.
        :return: the value of the requested attribute.
        """
        # The natural size = default size and the requested size = minimum size
        _attributes = {
            "height": self.get_preferred_height()[1],
            "width": self.get_preferred_width()[1],
        }

        return _attributes[attribute]

    def do_set_properties(self, **kwargs: Any) -> None:
        """Set the RAMSTKFrame properties.

        :Keyword Arguments:
            * *width* (int) -- width of the Gtk.Label() widget.  Default is
                190.
            * *height* (int) -- height of the Gtk.Label() widget.  Default is
                25.
            * *bold* (bool) -- boolean indicating whether text should be bold.
                Default is True.
            * *wrap* (bool) -- boolean indicating whether the label text should
                wrap or not.  Default is False.
            * *justify* (str) -- the justification type when the label wraps
                and contains more than one line.  Default is JUSTIFY_LEFT.
            * *tooltip* (str) -- the tooltip, if any, for the label.
                Default is an empty string.
        :return: None
        :rtype: None
        """
        super().do_set_properties(**kwargs)

        _bold = kwargs.get("bold", True)
        _justify = kwargs.get("justify", Gtk.Justification.LEFT)
        _wrap = kwargs.get("wrap", False)

        self.set_property("wrap", _wrap)
        self.set_property("justify", _justify)
        if _justify == Gtk.Justification.CENTER:
            self.set_xalign(0.5)
            self.set_yalign(0.5)
        elif _justify == Gtk.Justification.LEFT:
            self.set_xalign(0.05)
            self.set_yalign(0.5)
        else:
            self.set_xalign(0.99)
            self.set_yalign(0.5)

        if _bold:
            _text = self.get_property("label")
            _text = "<b>" + _text + "</b>"
            self.set_markup(_text)

    # pylint: disable=unused-argument
    def do_update(self, text: str, signal: str = "") -> None:
        """Update the text displayed by the label.

        :param text: the information to update the RAMSTKLabel() to display.
        :param signal: the name of the signal whose handler ID the RAMSTKLabel()
            needs to block.  Unused in this method, but required for compatibility.
        :return: None
        :rtype: None
        """
        self.set_markup("<span>" + text + "</span>")


def do_make_label_group(
    text: List[str], **kwargs: Dict[str, Any]
) -> Tuple[int, List[RAMSTKLabel]]:
    """Make and place a group of labels.

    The width of each label is set using a natural request.  This ensures the
    label doesn't cut off letters.  The maximum size of the labels is
    determined and used to set the left position of widget displaying the data
    described by the label.  This ensures everything lines up.  It also returns
    a list of y-coordinates indicating the placement of each label that is used
    to place the corresponding widget.

    :param text: a list containing the text for each label.
    :return: (_max_x, _lst_labels)
        the width of the label with the longest text and a list of the
        RAMSTKLabel() instances.
    :rtype: tuple of (integer, list of RAMSTKLabel())
    """
    _bold = kwargs.get("bold", True)
    _justify = kwargs.get("justify", Gtk.Justification.RIGHT)
    _wrap = kwargs.get("wrap", True)

    _lst_labels = []
    _max_x = 0

    _char_width = max([len(_label_text) for _label_text in text])

    # pylint: disable=unused-variable
    for __, _label_text in enumerate(text):
        _label = RAMSTKLabel(_label_text)
        _label.do_set_properties(
            bold=_bold, height=-1, justify=_justify, width=-1, wrap=_wrap
        )
        _label.set_width_chars(_char_width)
        _max_x = max(_max_x, _label.get_attribute("width"))
        _lst_labels.append(_label)

    return _max_x, _lst_labels



# pylint: disable=unused-import, wrong-import-position
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.design_electric.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Design Electric Package."""

# RAMSTK Local Imports
from .components.capacitor import CapacitorDesignElectricInputPanel
from .components.connection import ConnectionDesignElectricInputPanel
from .components.inductor import InductorDesignElectricInputPanel
from .components.integrated_circuit import ICDesignElectricInputPanel
from .components.meter import MeterDesignElectricInputPanel
from .components.miscellaneous import MiscDesignElectricInputPanel
from .components.relay import RelayDesignElectricInputPanel
from .components.resistor import ResistorDesignElectricInputPanel
from .components.semiconductor import SemiconductorDesignElectricInputPanel
from .components.switch import SwitchDesignElectricInputPanel
from .panel import (
    DesignElectricEnvironmentalInputPanel,
    DesignElectricStressInputPanel,
    DesignElectricStressResultPanel,
)



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord

class RAMSTKSimilarItemRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any
    __tablename__: str
    __table_args__: Any
    revision_id: Any
    hardware_id: Any
    change_description_1: Any
    change_description_2: Any
    change_description_3: Any
    change_description_4: Any
    change_description_5: Any
    change_description_6: Any
    change_description_7: Any
    change_description_8: Any
    change_description_9: Any
    change_description_10: Any
    change_factor_1: Any
    change_factor_2: Any
    change_factor_3: Any
    change_factor_4: Any
    change_factor_5: Any
    change_factor_6: Any
    change_factor_7: Any
    change_factor_8: Any
    change_factor_9: Any
    change_factor_10: Any
    environment_from_id: Any
    environment_to_id: Any
    function_1: Any
    function_2: Any
    function_3: Any
    function_4: Any
    function_5: Any
    similar_item_method_id: Any
    parent_id: Any
    quality_from_id: Any
    quality_to_id: Any
    result_1: Any
    result_2: Any
    result_3: Any
    result_4: Any
    result_5: Any
    temperature_from: Any
    temperature_to: Any
    user_blob_1: Any
    user_blob_2: Any
    user_blob_3: Any
    user_blob_4: Any
    user_blob_5: Any
    user_float_1: Any
    user_float_2: Any
    user_float_3: Any
    user_float_4: Any
    user_float_5: Any
    user_int_1: Any
    user_int_2: Any
    user_int_3: Any
    user_int_4: Any
    user_int_5: Any
    def get_attributes(self): ...



# -*- coding: utf-8 -*-
#
#       ramstk.analyses.models.milhdbk217f..Crystal.py is part of the RAMSTK
#       Project
#
# All rights reserved.
# Copyright 2007 - 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Crystal MIL-HDBK-217F Constants and Calculations Module."""

# Standard Library Imports
from typing import Any, Dict

PART_COUNT_LAMBDA_B = [
    0.032,
    0.096,
    0.32,
    0.19,
    0.51,
    0.38,
    0.54,
    0.70,
    0.90,
    0.74,
    0.016,
    0.42,
    1.0,
    16.0,
]
PART_COUNT_PI_Q = [1.0, 3.4]
PART_STRESS_PI_Q = [1.0, 2.1]
PI_E = [1.0, 3.0, 10.0, 6.0, 16.0, 12.0, 17.0, 22.0, 28.0, 23.0, 0.5, 13.0, 32.0, 500.0]


def calculate_part_count(**attributes: Dict[str, Any]) -> float:
    """Wrap get_part_count_lambda_b().

    This wrapper allows us to pass an attributes dict from a generic parts
    count function.

    :param attributes: the attributes for the crystal being calculated.
    :return: _base_hr; the list of base hazard rates.
    :rtype: float
    """
    return get_part_count_lambda_b(attributes["environment_active_id"])  # type: ignore


def calculate_part_stress(**attributes: Dict[str, Any]) -> Dict[str, Any]:
    """Calculate the part stress hazard rate for a crystal.

    This function calculates the MIL-HDBK-217F hazard rate using the part
    stress method.

    :param attributes: the attributes for the crystal being calculated.
    :return: attributes; the keyword argument (hardware attribute)
             dictionary with updated values.
    :rtype: dict
    """
    attributes["lambda_b"] = (
        0.013 * attributes["frequency_operating"] ** 0.23  # type: ignore
    )

    attributes["hazard_rate_active"] = (
        attributes["lambda_b"] * attributes["piQ"] * attributes["piE"]  # type: ignore
    )

    return attributes


def get_part_count_lambda_b(environment_active_id: int) -> float:
    """Retrieve the part count base hazard rate for a crystal.

    :param environment_active_id: the active environment identifier.
    :return: _base_hr; the part count base hazard rate for the active
        environment.
    :rtype: float
    :raise: IndexError if passed an unknown active environment ID.
    """
    return PART_COUNT_LAMBDA_B[environment_active_id - 1]



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.milhdbk217f.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKMilHdbkF Record Module."""

# Third Party Imports
from sqlalchemy import Column, Float, ForeignKey, Integer

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKMilHdbk217FRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_mil_hdbk_f table in RAMSTK Program database.

    This table shares a One-to-One relationship with ramstk_hardware.
    """

    __defaults__ = {
        "A1": 0.0,
        "A2": 0.0,
        "B1": 0.0,
        "B2": 0.0,
        "C1": 0.0,
        "C2": 0.0,
        "lambdaBD": 0.0,
        "lambdaBP": 0.0,
        "lambdaCYC": 0.0,
        "lambdaEOS": 0.0,
        "piA": 0.0,
        "piC": 0.0,
        "piCD": 0.0,
        "piCF": 0.0,
        "piCR": 0.0,
        "piCV": 0.0,
        "piCYC": 0.0,
        "piE": 0.0,
        "piF": 0.0,
        "piI": 0.0,
        "piK": 0.0,
        "piL": 0.0,
        "piM": 0.0,
        "piMFG": 0.0,
        "piN": 0.0,
        "piNR": 0.0,
        "piP": 0.0,
        "piPT": 0.0,
        "piQ": 0.0,
        "piR": 0.0,
        "piS": 0.0,
        "piT": 0.0,
        "piTAPS": 0.0,
        "piU": 0.0,
        "piV": 0.0,
    }
    __tablename__ = "ramstk_mil_hdbk_f"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        ForeignKey("ramstk_revision.fld_revision_id", ondelete="CASCADE"),
        nullable=False,
    )
    hardware_id = Column(
        "fld_hardware_id",
        Integer,
        ForeignKey("ramstk_hardware.fld_hardware_id", ondelete="CASCADE"),
        primary_key=True,
        nullable=False,
    )

    A1 = Column("fld_a_one", Float, default=__defaults__["A1"])
    A2 = Column("fld_a_two", Float, default=__defaults__["A2"])
    B1 = Column("fld_b_one", Float, default=__defaults__["B1"])
    B2 = Column("fld_b_two", Float, default=__defaults__["B2"])
    C1 = Column("fld_c_one", Float, default=__defaults__["C1"])
    C2 = Column("fld_c_two", Float, default=__defaults__["C2"])
    lambdaBD = Column("fld_lambda_bd", Float, default=__defaults__["lambdaBD"])
    lambdaBP = Column("fld_lambda_bp", Float, default=__defaults__["lambdaBP"])
    lambdaCYC = Column("fld_lambda_cyc", Float, default=__defaults__["lambdaCYC"])
    lambdaEOS = Column("fld_lambda_eos", Float, default=__defaults__["lambdaEOS"])
    piA = Column("fld_pi_a", Float, default=__defaults__["piA"])
    piC = Column("fld_pi_c", Float, default=__defaults__["piC"])
    piCD = Column("fld_pi_cd", Float, default=__defaults__["piCD"])
    piCF = Column("fld_pi_cf", Float, default=__defaults__["piCF"])
    piCR = Column("fld_pi_cr", Float, default=__defaults__["piCR"])
    piCV = Column("fld_pi_cv", Float, default=__defaults__["piCV"])
    piCYC = Column("fld_pi_cyc", Float, default=__defaults__["piCYC"])
    piE = Column("fld_pi_e", Float, default=__defaults__["piE"])
    piF = Column("fld_pi_f", Float, default=__defaults__["piF"])
    piI = Column("fld_pi_i", Float, default=__defaults__["piI"])
    piK = Column("fld_pi_k", Float, default=__defaults__["piK"])
    piL = Column("fld_pi_l", Float, default=__defaults__["piL"])
    piM = Column("fld_pi_m", Float, default=__defaults__["piM"])
    piMFG = Column("fld_pi_mfg", Float, default=__defaults__["piMFG"])
    piN = Column("fld_pi_n", Float, default=__defaults__["piN"])
    piNR = Column("fld_pi_nr", Float, default=__defaults__["piNR"])
    piP = Column("fld_pi_p", Float, default=__defaults__["piP"])
    piPT = Column("fld_pi_pt", Float, default=__defaults__["piPT"])
    piQ = Column("fld_pi_q", Float, default=__defaults__["piQ"])
    piR = Column("fld_pi_r", Float, default=__defaults__["piR"])
    piS = Column("fld_pi_s", Float, default=__defaults__["piS"])
    piT = Column("fld_pi_t", Float, default=__defaults__["piT"])
    piTAPS = Column("fld_pi_taps", Float, default=__defaults__["piTAPS"])
    piU = Column("fld_pi_u", Float, default=__defaults__["piU"])
    piV = Column("fld_pi_v", Float, default=__defaults__["piV"])

    # Define the relationships to other tables in the RAMSTK Program database.

    def get_attributes(self):
        """Retrieve the current values of RAMSTKMilHdbkF data model attributes.

        :return: {hardware_id, A2, A2, B1, B2, C1, C2, lambdaBD, lambdaBP,
                  lambdaCYC, lambdaEOS, piA, piC, piCD, piCF, piCR, piCV,
                  piCYC, piE, piF, piI, piK, piL, piM, piMFG, piN, piNR, piP,
                  piPT, piQ, piR, piS, piT, piTAPS, piU, piV} pairs.
        :rtype: dict
        """
        _attributes = {
            "hardware_id": self.hardware_id,
            "A1": self.A1,
            "A2": self.A2,
            "B1": self.B1,
            "B2": self.B2,
            "C1": self.C1,
            "C2": self.C2,
            "lambdaBD": self.lambdaBD,
            "lambdaBP": self.lambdaBP,
            "lambdaCYC": self.lambdaCYC,
            "lambdaEOS": self.lambdaEOS,
            "piA": self.piA,
            "piC": self.piC,
            "piCD": self.piCD,
            "piCF": self.piCF,
            "piCR": self.piCR,
            "piCV": self.piCV,
            "piCYC": self.piCYC,
            "piE": self.piE,
            "piF": self.piF,
            "piI": self.piI,
            "piK": self.piK,
            "piL": self.piL,
            "piM": self.piM,
            "piMFG": self.piMFG,
            "piN": self.piN,
            "piNR": self.piNR,
            "piP": self.piP,
            "piPT": self.piPT,
            "piQ": self.piQ,
            "piR": self.piR,
            "piS": self.piS,
            "piT": self.piT,
            "piTAPS": self.piTAPS,
            "piU": self.piU,
            "piV": self.piV,
        }

        return _attributes



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.program_status.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Program Status Record Model."""

# Standard Library Imports
from datetime import date

# Third Party Imports
from sqlalchemy import Column, Date, Float, ForeignKey, Integer

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKProgramStatusRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class represent table ramstk_program_status in RAMSTK Program database.

    This table shares a Many-to-One relationship with ramstk_revision.
    """

    __defaults__ = {
        "cost_remaining": 0.0,
        "date_status": date.today(),
        "time_remaining": 0.0,
    }
    __tablename__ = "ramstk_program_status"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        ForeignKey("ramstk_revision.fld_revision_id", ondelete="CASCADE"),
        nullable=False,
    )
    status_id = Column(
        "fld_status_id", Integer, primary_key=True, autoincrement=True, nullable=False
    )

    cost_remaining = Column("fld_cost_remaining", Float, default=0.0)
    date_status = Column("fld_date_status", Date, unique=True, default=date.today())
    time_remaining = Column("fld_time_remaining", Float, default=0.0)

    # Define the relationships to other tables in the RAMSTK Program database.

    def get_attributes(self):
        """Retrieve current values of RAMSTKProgramStatus model attributes.

        :return: {revision_id, cost_remaining, date_status, time_remaining}
            pairs.
        :rtype: dict
        """
        _attributes = {
            "revision_id": self.revision_id,
            "status_id": self.status_id,
            "cost_remaining": self.cost_remaining,
            "date_status": self.date_status,
            "time_remaining": self.time_remaining,
        }

        return _attributes



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.reliability.panel.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Reliability Panels."""

# Standard Library Imports
import locale
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKComboBox, RAMSTKEntry, RAMSTKFixedPanel


class ReliabilityInputPanel(RAMSTKFixedPanel):
    """Panel to display hazard rate inputs about the selected Hardware item.

    The widgets of a Reliability input panel are:

    :ivar cmbFailureDist: the RAMSTKComboBox() used to select and display the failure
        distribution for the selected hardware item.
    :ivar cmbHRMethod: the RAMSTKComboBox() used to select and display the method
        used to calculate the hazard rate for the selected hardware item.
    :ivar cmbHRType: the RAMSTKComboBox() used to select and display the
    :ivar txtAddAdjFactor: the RAMSTKEntry() used to input and display the
        hazard rate additive adjustment factor for the selected hardware item.
    :ivar txtFailLocation: the RAMSTKEntry() used to input and display the location
        parameter for the selected failure distribution.
    :ivar txtFailScale: the RAMSTKEntry() used to input and display the scale
        parameter for the selected failure distribution.
    :ivar txtFailShape: the RAMSTKEntry() used to input and display the shape
        parameter for the selected failure distribution.
    :ivar txtMultAdjFactor: the RAMSTKEntry() used to input and display the hazard
        rate multiplicative adjustment factor for the selected hardware item.
    :ivar txtSpecifiedHt: the RAMSTKEntry() used to input and display the specified
        hazard rate for the selected hardware item.
    :ivar txtSpecifiedHtVar: the RAMSTKEntry() used to input and display the variance of
        the specified hazard rate for the selected hardware item.
    :ivar txtSpecifiedMTBF: the RAMSTKEntry() used to input and display the specified
        MTBF for the selected hardware item.
    :ivar txtSpecifiedMTBFVar: the RAMSTKEntry() used to input and display the
        variance of the specified MTBF for the selected hardware item.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "succeed_get_reliability_attributes"
    _tag: str = "reliability"
    _title: str = _("Reliability Assessment Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Assessment Input panel."""
        super().__init__()

        # Initialize widgets.
        self.cmbFailureDist: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbHRMethod: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbHRType: RAMSTKComboBox = RAMSTKComboBox()
        self.txtAddAdjFactor: RAMSTKEntry = RAMSTKEntry()
        self.txtFailLocation: RAMSTKEntry = RAMSTKEntry()
        self.txtFailScale: RAMSTKEntry = RAMSTKEntry()
        self.txtFailShape: RAMSTKEntry = RAMSTKEntry()
        self.txtMultAdjFactor: RAMSTKEntry = RAMSTKEntry()
        self.txtSpecifiedHt: RAMSTKEntry = RAMSTKEntry()
        self.txtSpecifiedHtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtSpecifiedMTBF: RAMSTKEntry = RAMSTKEntry()
        self.txtSpecifiedMTBFVar: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dict instance attributes.

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "hazard_rate_type_id": [
                17,
                self.cmbHRType,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _(
                        "The type of reliability assessment for the selected hardware "
                        "item."
                    ),
                    "width": 200,
                },
                _("Assessment Type:"),
                "gint",
            ],
            "hazard_rate_method_id": [
                11,
                self.cmbHRMethod,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _(
                        "The assessment method to use for the selected hardware item."
                    ),
                    "width": 200,
                },
                _("Assessment Method:"),
                "gint",
            ],
            "hazard_rate_specified": [
                16,
                self.txtSpecifiedHt,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The stated hazard rate."),
                    "width": 125,
                },
                _("Stated Hazard Rate [h(t)]:"),
                "gfloat",
            ],
            "hr_specified_variance": [
                22,
                self.txtSpecifiedHtVar,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The variance of the stated hazard rate."),
                    "width": 125,
                },
                _("Stated h(t) Variance:"),
                "gfloat",
            ],
            "mtbf_specified": [
                26,
                self.txtSpecifiedMTBF,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The stated mean time between failure (MTBF)."),
                    "width": 125,
                },
                _("Stated MTBF:"),
                "gfloat",
            ],
            "mtbf_specified_variance": [
                30,
                self.txtSpecifiedMTBFVar,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The variance of the stated mean time between failure (MTBF)."
                    ),
                    "width": 125,
                },
                _("Stated MTBF Variance:"),
                "gfloat",
            ],
            "failure_distribution_id": [
                7,
                self.cmbFailureDist,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _(
                        "The statistical failure distribution of the selected hardware "
                        "item."
                    ),
                    "width": 200,
                },
                _("Failure Distribution:"),
                "gint",
            ],
            "scale_parameter": [
                39,
                self.txtFailScale,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The scale parameter of the statistical failure distribution."
                    ),
                    "width": 125,
                },
                _("Scale Parameter:"),
                "gfloat",
            ],
            "shape_parameter": [
                40,
                self.txtFailShape,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The shape parameter of the statistical failure distribution."
                    ),
                    "width": 125,
                },
                _("Shape Parameter:"),
                "gfloat",
            ],
            "location_parameter": [
                24,
                self.txtFailLocation,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The location parameter of the statistical failure "
                        "distribution."
                    ),
                    "width": 125,
                },
                _("Location Parameter:"),
                "gfloat",
            ],
            "add_adj_factor": [
                2,
                self.txtAddAdjFactor,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "An adjustment factor to add to the assessed hazard rate."
                    ),
                    "width": 125,
                },
                _("Additive Adjustment Factor:"),
                "gfloat",
            ],
            "mult_adj_factor": [
                31,
                self.txtMultAdjFactor,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                1.0,
                {
                    "tooltip": _(
                        "An adjustment factor to multiply the assessed hazard rate."
                    ),
                    "width": 125,
                },
                _("Multiplicative Adjustment Factor:"),
                "gfloat",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.

    def do_load_hr_distributions(self, distributions: List[str]) -> None:
        """Load the hazard rate distribution RAMSTKComboBox().

        :param distributions: the list of s-distribution names RAMSTK currently
            supports.
        :return: None
        """
        self.cmbFailureDist.do_load_combo(entries=distributions)

    def do_load_hr_methods(self, methods: List[str]) -> None:
        """Load the hazard rate method RAMSTKComboBox().

        The hazard rate methods are:

            * MIL-HDBK-217F Parts Count
            * MIL-HDBK-217F Parts Stress
            * NSWC-11

        :param methods: the list of methods for assessing the hazard rate.
        :return: None
        """
        self.cmbHRMethod.do_load_combo(entries=methods)

    def do_load_hr_types(self, hr_types: List[str]) -> None:
        """Load the hazard rate type RAMSTKComboBox().

        The hazard rate types are:

            * Assessed
            * Defined, Hazard Rate
            * Defined, MTBF
            * Defined, Distribution

        :param hr_types: the types (or ways) of establishing the hazard rate
            for a hardware item.
        :return: None
        """
        self.cmbHRType.do_load_combo(entries=hr_types)

    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set certain widgets sensitive or insensitive.

        This method will set the sensitivity of various widgets depending on
        the hazard rate assessment type selected.

        :return: None
        :rtype: None
        """
        self._do_set_sensitive_assessed(attributes["hazard_rate_type_id"])
        self._do_set_sensitive_specified_ht(attributes["hazard_rate_type_id"])
        self._do_set_sensitive_specified_mtbf(attributes["hazard_rate_type_id"])
        self._do_set_sensitive_specified_distribution(attributes["hazard_rate_type_id"])

    def _do_set_sensitive_assessed(self, type_id: int) -> None:
        """Set the widgets used in handbook assessments sensitive.

        :param type_id: the hazard rate type (source).
        :return: None
        :rtype: None
        """
        if type_id == 1:  # Assessed hazard rate using handbook models.
            self.cmbFailureDist.set_sensitive(False)
            self.cmbHRMethod.set_sensitive(True)
            self.txtFailLocation.set_sensitive(False)
            self.txtFailScale.set_sensitive(False)
            self.txtFailShape.set_sensitive(False)
            self.txtSpecifiedHt.set_sensitive(False)
            self.txtSpecifiedHtVar.set_sensitive(False)
            self.txtSpecifiedMTBF.set_sensitive(False)
            self.txtSpecifiedMTBFVar.set_sensitive(False)

    def _do_set_sensitive_specified_ht(self, type_id: int) -> None:
        """Set the widgets used for specifying a hazard rate sensitive.

        :param type_id: the hazard rate type (source).
        :return: None
        :rtype: None
        """
        if type_id == 2:  # User specified hazard rate.
            self.cmbFailureDist.set_sensitive(False)
            self.cmbHRMethod.set_sensitive(False)
            self.txtFailLocation.set_sensitive(False)
            self.txtFailScale.set_sensitive(False)
            self.txtFailShape.set_sensitive(False)
            self.txtSpecifiedHt.set_sensitive(True)
            self.txtSpecifiedHtVar.set_sensitive(True)
            self.txtSpecifiedMTBF.set_sensitive(False)
            self.txtSpecifiedMTBFVar.set_sensitive(False)

    def _do_set_sensitive_specified_mtbf(self, type_id: int) -> None:
        """Set the widgets used for specifying an MTBF sensitive.

        :param type_id: the hazard rate type (source).
        :return: None
        :rtype: None
        """
        if type_id == 3:  # User specified MTBF.
            self.cmbFailureDist.set_sensitive(False)
            self.cmbHRMethod.set_sensitive(False)
            self.txtFailLocation.set_sensitive(False)
            self.txtFailScale.set_sensitive(False)
            self.txtFailShape.set_sensitive(False)
            self.txtSpecifiedHt.set_sensitive(False)
            self.txtSpecifiedHtVar.set_sensitive(False)
            self.txtSpecifiedMTBF.set_sensitive(True)
            self.txtSpecifiedMTBFVar.set_sensitive(True)

    def _do_set_sensitive_specified_distribution(self, type_id: int) -> None:
        """Set widgets used for specifying a failure distribution sensitive.

        :param type_id: the hazard rate type (source).
        :return: None
        :rtype: None
        """
        if type_id == 4:
            self.cmbFailureDist.set_sensitive(True)
            self.cmbHRMethod.set_sensitive(False)
            self.txtFailLocation.set_sensitive(True)
            self.txtFailScale.set_sensitive(True)
            self.txtFailShape.set_sensitive(True)
            self.txtSpecifiedHt.set_sensitive(False)
            self.txtSpecifiedHtVar.set_sensitive(False)
            self.txtSpecifiedMTBF.set_sensitive(False)
            self.txtSpecifiedMTBFVar.set_sensitive(False)


class ReliabilityResultsPanel(RAMSTKFixedPanel):
    """Panel to display reliability results for the selected Hardware item."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "succeed_get_reliability_attributes"
    _tag: str = "reliability"
    _title: str = _("Reliability Assessment Results")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Reliability Results panel."""
        super().__init__()

        # Initialize widgets.
        self.txtActiveHt: RAMSTKEntry = RAMSTKEntry()
        self.txtActiveHtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtDormantHt: RAMSTKEntry = RAMSTKEntry()
        self.txtDormantHtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtLogisticsHt: RAMSTKEntry = RAMSTKEntry()
        self.txtLogisticsHtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtLogisticsMTBF: RAMSTKEntry = RAMSTKEntry()
        self.txtLogisticsMTBFVar: RAMSTKEntry = RAMSTKEntry()
        self.txtLogisticsRt: RAMSTKEntry = RAMSTKEntry()
        self.txtLogisticsRtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionHt: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionHtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionMTBF: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionMTBFVar: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionRt: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionRtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtPercentHt: RAMSTKEntry = RAMSTKEntry()
        self.txtSoftwareHt: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dict instance attributes.

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "hazard_rate_active": [
                8,
                self.txtActiveHt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the active failure intensity for the selected "
                        "hardware item."
                    ),
                    "width": 125,
                },
                _("Active Failure Intensity [\u03BB(t)]:"),
                "gfloat",
            ],
            "hr_active_variance": [
                18,
                self.txtActiveHtVar,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "Displays the variance on the active failure intensity "
                        "for the selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "hazard_rate_dormant": [
                9,
                self.txtDormantHt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the dormant failure intensity for the "
                        "selected hardware item."
                    ),
                    "width": 125,
                },
                _("Dormant \u03BB(t):"),
                "gfloat",
            ],
            "hr_dormant_variance": [
                19,
                self.txtDormantHtVar,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "Displays the variance on the dormant failure intensity "
                        "for the selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "hazard_rate_software": [
                15,
                self.txtSoftwareHt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the software failure intensity for the "
                        "selected hardware item."
                    ),
                    "width": 125,
                },
                _("Software \u03BB(t):"),
                "gfloat",
            ],
            "hazard_rate_logistics": [
                10,
                self.txtLogisticsHt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the logistics failure intensity for the "
                        "selected hardware item.  This is the sum of the "
                        "active, dormant, and software hazard rates."
                    ),
                    "width": 125,
                },
                _("Logistics \u03BB(t):"),
                "gfloat",
            ],
            "hr_logistics_variance": [
                20,
                self.txtLogisticsHtVar,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "Displays the variance on the logistics failure "
                        "intensity for the selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "hazard_rate_mission": [
                12,
                self.txtMissionHt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the mission failure intensity for the "
                        "selected hardware item."
                    ),
                    "width": 125,
                },
                _("Mission \u03BB(t):"),
                "gfloat",
            ],
            "hr_mission_variance": [
                21,
                self.txtMissionHtVar,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the variance on the mission failure "
                        "intensity for the selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "hazard_rate_percent": [
                14,
                self.txtPercentHt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the percentage of the system failure "
                        "intensity the selected hardware item represents."
                    ),
                    "width": 125,
                },
                _("Percent \u03BB(t):"),
                "gfloat",
            ],
            "mtbf_logistics": [
                25,
                self.txtLogisticsMTBF,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the logistics mean time between failure "
                        "(MTBF) for the selected hardware item."
                    ),
                    "width": 125,
                },
                _("Logistics MTBF:"),
                "gfloat",
            ],
            "mtbf_logistics_variance": [
                28,
                self.txtLogisticsMTBFVar,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the variance on the logistics MTBF for the "
                        "selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "mtbf_mission": [
                26,
                self.txtMissionMTBF,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the mission mean time between failure (MTBF) "
                        "for the selected hardware item."
                    ),
                    "width": 125,
                },
                _("Mission MTBF:"),
                "gfloat",
            ],
            "mtbf_mission_variance": [
                29,
                self.txtMissionMTBFVar,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the variance on the mission MTBF for the selected "
                        "hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "reliability_logistics": [
                35,
                self.txtLogisticsRt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the logistics reliability for the selected "
                        "hardware item."
                    ),
                    "width": 125,
                },
                _("Logistics Reliability [R(t)]:"),
                "gfloat",
            ],
            "reliability_log_variance": [
                37,
                self.txtLogisticsRtVar,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the variance on the logistics reliability "
                        "for the selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "reliability_mission": [
                36,
                self.txtMissionRt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the mission reliability for the selected "
                        "hardware item."
                    ),
                    "width": 125,
                },
                _("Mission R(t):"),
                "gfloat",
            ],
            "reliability_miss_variance": [
                38,
                self.txtMissionRtVar,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the variance on the mission reliability for "
                        "the selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        super().do_set_properties()
        super().do_make_panel()
        self.__do_nudge_widgets()

        # Subscribe to PyPubSub messages.

    def _do_load_entries(self, attributes: Dict[str, Any]) -> None:
        """Load contents of the RAMSTKEntry() widgets.

        This method ensures results RAMSTKEntry() widgets are set insensitive and
        loads the contents.  The PyPubSub subscriber is in the metaclass.

        :return: None
        :rtype: None
        """
        self._do_load_entries_hazard_rate(attributes)
        self._do_load_entries_mtbf(attributes)
        self._do_load_entries_reliability(attributes)

    def _do_load_entries_hazard_rate(self, attributes: Dict[str, Any]) -> None:
        """Load contents of the hazard rate RAMSTKEntry().

        :return: None
        :rtype: None
        """
        self.txtActiveHt.do_update(
            str(self.fmt.format(attributes["hazard_rate_active"] or 0.0)),
            signal="changed",
        )
        self.txtActiveHtVar.do_update(
            str(self.fmt.format(attributes["hr_active_variance"] or 0.0)),
            signal="changed",
        )
        self.txtDormantHt.do_update(
            str(self.fmt.format(attributes["hazard_rate_dormant"] or 0.0)),
            signal="changed",
        )
        self.txtDormantHtVar.do_update(
            str(self.fmt.format(attributes["hr_dormant_variance"] or 0.0)),
            signal="changed",
        )
        self.txtSoftwareHt.do_update(
            str(self.fmt.format(attributes["hazard_rate_software"] or 0.0)),
            signal="changed",
        )
        self.txtLogisticsHt.do_update(
            str(self.fmt.format(attributes["hazard_rate_logistics"] or 0.0)),
            signal="changed",
        )
        self.txtLogisticsHtVar.do_update(
            str(self.fmt.format(attributes["hr_logistics_variance"] or 0.0)),
            signal="changed",
        )
        self.txtMissionHt.do_update(
            str(self.fmt.format(attributes["hazard_rate_mission"] or 0.0)),
            signal="changed",
        )
        self.txtMissionHtVar.do_update(
            str(self.fmt.format(attributes["hr_mission_variance"] or 0.0)),
            signal="changed",
        )
        self.txtPercentHt.do_update(
            str(self.fmt.format(attributes["hazard_rate_percent"] or 0.0)),
            signal="changed",
        )

    def _do_load_entries_mtbf(self, attributes: Dict[str, Any]) -> None:
        """Load contents of MTBF RAMSTKEntry().

        :return: None
        :rtype: None
        """
        self.txtLogisticsMTBF.do_update(
            str(self.fmt.format(attributes["mtbf_logistics"] or 0.0)),
            signal="changed",
        )
        self.txtLogisticsMTBFVar.do_update(
            str(self.fmt.format(attributes["mtbf_logistics_variance"] or 0.0)),
            signal="changed",
        )
        self.txtMissionMTBF.do_update(
            str(self.fmt.format(attributes["mtbf_mission"] or 0.0)),
            signal="changed",
        )
        self.txtMissionMTBFVar.do_update(
            str(self.fmt.format(attributes["mtbf_mission_variance"] or 0.0)),
            signal="changed",
        )

    def _do_load_entries_reliability(self, attributes: Dict[str, Any]) -> None:
        """Load contents of reliability RAMSTKEntry().

        :return: None
        :rtype: None
        """
        self.txtLogisticsRt.do_update(
            str(self.fmt.format(attributes["reliability_logistics"] or 1.0)),
            signal="changed",
        )
        self.txtLogisticsRtVar.do_update(
            str(self.fmt.format(attributes["reliability_log_variance"] or 0.0)),
            signal="changed",
        )
        self.txtMissionRt.do_update(
            str(self.fmt.format(attributes["reliability_mission"] or 1.0)),
            signal="changed",
        )
        self.txtMissionRtVar.do_update(
            str(self.fmt.format(attributes["reliability_miss_variance"] or 0.0)),
            signal="changed",
        )

    def __do_nudge_widgets(self) -> None:
        """Adjust widgets from their default positions.

        :return: None
        :rtype: None
        """
        _lst_labels: List[object] = []
        _x_pos: List[int] = [0, self.txtActiveHt.get_preferred_size()[0].width + 5]
        _y_pos: List[int] = []
        _n_rows: int = 0

        _fixed = self.get_children()[0].get_children()[0].get_children()[0]
        _widgets = (
            self.get_children()[0].get_children()[0].get_children()[0].get_children()
        )

        for _widget in _widgets[::2]:
            _y_pos.append(_fixed.child_get_property(_widget, "y"))
            if _widget.get_text() != "":
                _lst_labels.append(_widget)
                _x_pos[0] = max(_x_pos[0], _widget.get_preferred_size()[0].width)
                _n_rows += 1

        _x_pos[0] += 10
        _x_pos[1] = _x_pos[0] + _x_pos[1] + 5
        for _idx, _pos in enumerate(_y_pos[:_n_rows]):
            _fixed.move(_lst_labels[_idx], 5, _pos)

        _fixed.move(self.txtActiveHtVar, _x_pos[1], _y_pos[0])
        _fixed.move(self.txtDormantHt, _x_pos[0], _y_pos[1])
        _fixed.move(self.txtDormantHtVar, _x_pos[1], _y_pos[1])
        _fixed.move(self.txtSoftwareHt, _x_pos[0], _y_pos[2])
        _fixed.move(self.txtLogisticsHt, _x_pos[0], _y_pos[3])
        _fixed.move(self.txtLogisticsHtVar, _x_pos[1], _y_pos[3])
        _fixed.move(self.txtMissionHt, _x_pos[0], _y_pos[4])
        _fixed.move(self.txtMissionHtVar, _x_pos[1], _y_pos[4])
        _fixed.move(self.txtPercentHt, _x_pos[0], _y_pos[5])
        _fixed.move(self.txtLogisticsMTBF, _x_pos[0], _y_pos[6])
        _fixed.move(self.txtLogisticsMTBFVar, _x_pos[1], _y_pos[6])
        _fixed.move(self.txtMissionMTBF, _x_pos[0], _y_pos[7])
        _fixed.move(self.txtMissionMTBFVar, _x_pos[1], _y_pos[7])
        _fixed.move(self.txtLogisticsRt, _x_pos[0], _y_pos[8])
        _fixed.move(self.txtLogisticsRtVar, _x_pos[1], _y_pos[8])
        _fixed.move(self.txtMissionRt, _x_pos[0], _y_pos[9])
        _fixed.move(self.txtMissionRtVar, _x_pos[1], _y_pos[9])


class AvailabilityResultsPanel(RAMSTKFixedPanel):
    """Panel to display availability results for the selected Hardware item."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "succeed_get_reliability_attributes"
    _tag: str = "reliability"
    _title: str = _("Availability Assessment Results")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Availability Results panel."""
        super().__init__()

        # Initialize widgets.
        self.txtCostFailure: RAMSTKEntry = RAMSTKEntry()
        self.txtCostHour: RAMSTKEntry = RAMSTKEntry()
        self.txtLogisticsAt: RAMSTKEntry = RAMSTKEntry()
        self.txtLogisticsAtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionAt: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionAtVar: RAMSTKEntry = RAMSTKEntry()
        self.txtPartCount: RAMSTKEntry = RAMSTKEntry()
        self.txtTotalCost: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dict instance attributes.

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "availability_logistics": [
                3,
                self.txtLogisticsAt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the logistics availability for the selected "
                        "hardware item."
                    ),
                    "width": 125,
                },
                _("Logistics Availability [A(t)]:"),
                "gfloat",
            ],
            "avail_log_variance": [
                5,
                self.txtLogisticsAtVar,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "Displays the variance on the logistics availability for the "
                        "selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "availability_mission": [
                4,
                self.txtMissionAt,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the mission availability for the selected "
                        "hardware item."
                    ),
                    "width": 125,
                },
                _("Mission A(t):"),
                "gfloat",
            ],
            "avail_mis_variance": [
                6,
                self.txtMissionAtVar,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "Displays the variance on the mission availability for the "
                        "selected hardware item."
                    ),
                    "width": 125,
                },
                "",
                "gfloat",
            ],
            "total_cost": [
                31,
                self.txtTotalCost,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the total cost of the selected hardware item."
                    ),
                    "width": 125,
                },
                _("Total Cost:"),
                "gfloat",
            ],
            "cost_failure": [
                8,
                self.txtCostFailure,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the cost per failure of the selected hardware item."
                    ),
                    "width": 125,
                },
                _("Cost/Failure:"),
                "gfloat",
            ],
            "cost_hour": [
                9,
                self.txtCostHour,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "Displays the failure cost per life time hour for the "
                        "selected hardware item."
                    ),
                    "width": 125,
                },
                _("Cost/Hour:"),
                "gfloat",
            ],
            "total_part_count": [
                32,
                self.txtPartCount,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _(
                        "Displays the total part count for the selected hardware item."
                    ),
                    "width": 125,
                },
                _("Total # of Parts:"),
                "gint",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        # Make a fixed type panel.
        super().do_set_properties()
        super().do_make_panel()
        self.__do_nudge_widgets()

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self._do_load_entries_hardware,
            "succeed_get_hardware_attributes",
        )

    def _do_load_entries(self, attributes: Dict[str, Any]) -> None:
        """Load contents of the RAMSTKEntry() widgets.

        This method ensures results RAMSTKEntry() widgets are set insensitive and
        loads the contents.  The PyPubSub subscriber is in the metaclass.

        :return: None
        :rtype: None
        """
        self.txtLogisticsAt.do_update(
            str(self.fmt.format(attributes["availability_logistics"] or 1.0)),
            signal="changed",
        )
        self.txtLogisticsAtVar.do_update(
            str(self.fmt.format(attributes["avail_log_variance"] or 0.0)),
            signal="changed",
        )
        self.txtMissionAt.do_update(
            str(self.fmt.format(attributes["availability_mission"] or 1.0)),
            signal="changed",
        )
        self.txtMissionAtVar.do_update(
            str(self.fmt.format(attributes["avail_mis_variance"] or 0.0)),
            signal="changed",
        )

    def _do_load_entries_hardware(self, attributes: Dict[str, Any]) -> None:
        """Load contents of the RAMSTKEntry() widgets.

        This method ensures results RAMSTKEntry() widgets are set insensitive and
        loads the contents.  The PyPubSub subscriber is in the metaclass.

        :return: None
        :rtype: None
        """
        self.txtTotalCost.do_update(
            str(locale.currency(attributes["total_cost"])),
            signal="changed",
        )
        self.txtCostFailure.do_update(
            str(locale.currency(attributes["cost_failure"])),
            signal="changed",
        )
        self.txtCostHour.do_update(
            str(locale.currency(attributes["cost_hour"])),
            signal="changed",
        )
        self.txtPartCount.do_update(
            str("{0:d}".format(attributes["total_part_count"])),
            signal="changed",
        )

    def __do_nudge_widgets(self) -> None:
        """Adjust widgets from their default positions.

        :return: None
        :rtype: None
        """
        _lst_labels: List[object] = []
        _x_pos: List[int] = [0, self.txtLogisticsAt.get_preferred_size()[0].width + 5]
        _y_pos: List[int] = []
        _n_rows: int = 0

        _fixed = self.get_children()[0].get_children()[0].get_children()[0]
        _widgets = (
            self.get_children()[0].get_children()[0].get_children()[0].get_children()
        )

        for _widget in _widgets[::2]:
            _y_pos.append(_fixed.child_get_property(_widget, "y"))
            if _widget.get_text() != "":
                _lst_labels.append(_widget)
                _x_pos[0] = max(_x_pos[0], _widget.get_preferred_size()[0].width)
                _n_rows += 1

        _x_pos[0] += 10
        _x_pos[1] = _x_pos[0] + _x_pos[1] + 5
        for _idx, _pos in enumerate(_y_pos[:_n_rows]):
            _fixed.move(_lst_labels[_idx], 5, _pos)

        _fixed.move(self.txtLogisticsAtVar, _x_pos[1], _y_pos[0])
        _fixed.move(self.txtMissionAt, _x_pos[0], _y_pos[1])
        _fixed.move(self.txtMissionAtVar, _x_pos[1], _y_pos[1])
        _fixed.move(self.txtTotalCost, _x_pos[0], _y_pos[2])
        _fixed.move(self.txtCostFailure, _x_pos[0], _y_pos[3])
        _fixed.move(self.txtCostHour, _x_pos[0], _y_pos[4])
        _fixed.move(self.txtPartCount, _x_pos[0], _y_pos[5])



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKMethod.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKMethod Table Module."""

# Third Party Imports
from sqlalchemy import Column, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKMethod(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to representramstk_method in the RAMSTK Common database."""

    __defaults__ = {
        "description": "Method Description",
        "method_type": "unknown",
        "name": "Method Name",
    }
    __tablename__ = "ramstk_method"
    __table_args__ = {"extend_existing": True}

    method_id = Column(
        "fld_method_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    name = Column("fld_name", String(256), default=__defaults__["name"])
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    method_type = Column(
        "fld_method_type", String(256), default=__defaults__["method_type"]
    )

    def get_attributes(self):
        """Retrieve current values of the RAMSTKMethod data model attributes.

        :return: {method_id, name, description, method_type} pairs.
        :rtype: dict
        """
        _attributes = {
            "method_id": self.method_id,
            "name": self.name,
            "description": self.description,
            "method_type": self.method_type,
        }

        return _attributes
